/**
*   Docutain SDK React Native
*   Copyright (c) INFOSOFT Informations- und Dokumentationssysteme GmbH. All rights reserved.
*
*   Docutain SDK React Native is a commercial product and requires a license.
*   Details found in the LICENSE file in the root directory of this source tree.
*/

package com.docutainsdk

import android.app.Activity
import android.content.Intent
import android.net.Uri
import android.util.Log
import com.facebook.react.bridge.ActivityEventListener
import com.facebook.react.bridge.BaseActivityEventListener
import com.facebook.react.bridge.Promise
import com.facebook.react.bridge.ReactApplicationContext
import com.facebook.react.bridge.ReactContextBaseJavaModule
import com.facebook.react.bridge.ReactMethod
import com.facebook.react.bridge.ReadableMap
import com.facebook.react.bridge.ReadableArray
import com.facebook.react.bridge.ReadableType
import de.docutain.sdk.Document
import de.docutain.sdk.DocutainSDK
import de.docutain.sdk.Logger
import de.docutain.sdk.dataextraction.AnalyzeConfiguration
import de.docutain.sdk.dataextraction.DocumentDataReader
import de.docutain.sdk.ui.DocumentScanner
import de.docutain.sdk.ui.DocumentScannerConfiguration
import de.docutain.sdk.ui.DocutainColor
import de.docutain.sdk.ui.ScanFilter
import de.docutain.sdk.ui.Source
import java.io.File
import java.util.Base64

class DocutainSdkModule(reactContext: ReactApplicationContext) :
  ReactContextBaseJavaModule(reactContext) {

  private val logTag = "DocutainSdk"
  private val SCAN_DOCUMENT_RESULT_CODE = 1
  private var callerContext: ReactApplicationContext;
  private var scanPromise : Promise? = null


  private val mActivityEventListener: ActivityEventListener = object : BaseActivityEventListener() {
    override fun onActivityResult(activity: Activity?, requestCode: Int, resultCode: Int, intent: Intent?) {
      Log.v(logTag, "onActivityResult" )
      if (requestCode == SCAN_DOCUMENT_RESULT_CODE) {
        if (scanPromise != null) {
          scanPromise!!.resolve(if(resultCode == Activity.RESULT_OK) true else false)
          scanPromise = null
          Log.v(logTag, if(resultCode == Activity.RESULT_OK) "scannen finished" else "scannen canceled" )
        }
      }
    }
  }

  init   {
    callerContext = reactContext
    callerContext.addActivityEventListener(mActivityEventListener)
  }

  override fun getName(): String {
    return NAME
  }

  @ReactMethod
  fun initSDK(licenseKey: String, promise: Promise) {
    Log.v(logTag, "InitSDK Start")

    if(!DocutainSDK.initSDK(callerContext.getApplicationContext() as android.app.Application, licenseKey)){
      //init of Docutain SDK failed, get the last error message
      Log.e(logTag,"Initialization of the Docutain SDK failed: ${DocutainSDK.getLastError()}")
      //your logic to deactivate access to SDK functionality
      if(licenseKey == "YOUR_LICENSE_KEY_HERE"){
        licenseKeyMissing = true
      }
      promise.resolve(false);
      return;
    }
    else
    {
      Log.d(logTag, "initSDK Ok: $licenseKey")
    }
    promise.resolve(true);
  }

  @ReactMethod
  fun deleteTempFiles(deleteTraceFileContent: Boolean ,promise: Promise) {
    Log.v(logTag, "deleteTempFiles")
    try {
      val rc = DocutainSDK.deleteTempFiles(deleteTraceFileContent)
      promise.resolve(rc)
    }
    catch(ex:Exception )
    {
      Log.e(logTag, "deleteTempFiles Exception"+ ex)
      promise.reject(ex);
    }
  }

  @ReactMethod
  fun getLastError( promise: Promise) {
    Log.v(logTag, "getLastError")
    try {
      val text = DocutainSDK.getLastError()
      promise.resolve(text)
    }
    catch(ex:Exception )
    {
      Log.e(logTag, "getLastError Exception"+ ex);
      promise.reject(ex);
    }
  }

  @ReactMethod
  fun setLogLevel(LogLevel: String,  promise: Promise) {
    Log.v(logTag, "setLogLevel:" + LogLevel)
    try {
      val logLevel: Logger.Level = getLogLevelFromString(LogLevel);
      Log.d(logTag, "setLogLevel " + logLevel)
      Logger.setLogLevel(logLevel);
      promise.resolve(true)
    }
    catch(ex:Exception )
    {
      Log.e(logTag, "setLogLevel Exception"+ ex)
      promise.reject(ex);
    }
  }
  @ReactMethod
  fun getTraceFile( promise: Promise) {
    Log.v(logTag, "getTraceFile")
    try {
      val text = Logger.getTraceFile().getPath();
      promise.resolve(text)
    }
    catch(ex:Exception )
    {
      Log.e(logTag, "getLastError Exception"+ ex)
      promise.reject(ex);
    }
  }

  @ReactMethod
  fun setAnalyzeConfiguration(config: ReadableMap, promise: Promise) {
    Log.v(logTag, "setAnalyzeConfiguration")
    val analyzeConfig = AnalyzeConfiguration()
    mapAnalyzeConfiguration(config, analyzeConfig);
    Log.v(logTag, "analyzeConfig.readBIC:" + analyzeConfig.readBIC)
    if(!DocumentDataReader.setAnalyzeConfiguration(analyzeConfig)){
      Log.e(logTag,"Setting AnalyzeConfiguration failed: ${DocutainSDK.getLastError()}")
      promise.resolve(false);
    }
    else {
      promise.resolve(true);
    }
  }

  @ReactMethod
  fun scanDocument(options: ReadableMap, promise: Promise) {
    Log.v(logTag, "scanDocument Start")
    try {
      val currentActivity: Activity? = getCurrentActivity()
      if (currentActivity == null) {
        promise.resolve(false)
        return;
      }
      scanPromise = promise
      val nativeScanConfig = DocumentScannerConfiguration()
      mapDocumentScannerConfiguration(options, nativeScanConfig);
      Log.d(logTag, "allowCaptureModeSetting:" + nativeScanConfig.allowCaptureModeSetting)
      val intent = DocumentScanner.newScanIntent(callerContext.getApplicationContext(), nativeScanConfig)
      currentActivity.startActivityForResult(intent, SCAN_DOCUMENT_RESULT_CODE)
      Log.v(logTag, "scanDocument start OK")
    }
    catch(ex:Exception )
    {
      Log.e(logTag, "scanDocument Exception"+ ex)
      promise.reject(ex);
    }
  }

  @ReactMethod
  fun loadFile(filepath: String, promise: Promise) {
    Log.v(logTag, "loadFile:" + filepath)
    try {
      val fileUri:Uri = Uri.parse(filepath)
      val rc = DocumentDataReader.loadFile(fileUri)
      Log.d(logTag, "loadFile rc:" + rc)
      promise.resolve(rc)
    }
    catch(ex:Exception )
    {
      Log.e(logTag, "loadFile Exception"+ ex)
      promise.reject(ex);
    }
  }

  @ReactMethod
  fun writePDF(filepath: String, overWrite: Boolean, PageFormat: String, promise: Promise) {
    Log.v(logTag, "PageFormat:" + PageFormat)
    try {
      val PDFFormat: Document.PDFPageFormat = getPDFPageFormatFromString(PageFormat);
      Log.d(logTag, "PDFFormat:" + PDFFormat)

      val file = File(filepath);
      val rc = Document.writePDF(file,overWrite, PDFFormat)
      promise.resolve(rc?.absolutePath)
    }
    catch(ex:Exception )
    {
      Log.e(logTag, "writePDF Exception"+ ex)
      promise.reject(ex);
    }
  }

  @ReactMethod
  fun writeImage(pageNumber: Int, filepath: String, promise: Promise) {
    try {
      val file = File(filepath);
      val rc = Document.writeImage(pageNumber, file)
      promise.resolve(rc?.absolutePath)
    }
    catch(ex:Exception )
    {
      Log.e(logTag, "writeImage Exception"+ ex)
      promise.reject(ex);
    }
  }

  @ReactMethod
  fun getImageBytes(pageNumber: Int, pageSourceType: String, promise: Promise) {
    try {
      val sourceType: Document.PageSourceType = getPageSourceTypeFromString(pageSourceType);
      val bytes = Document.getImageBytes(pageNumber, sourceType)
      if(bytes == null){
        promise.resolve("")
      } else{        
        promise.resolve(Base64.getEncoder().encodeToString(bytes))
      }
    }
    catch(ex:Exception )
    {
      Log.e(logTag, "writeImage Exception"+ ex)
      promise.reject(ex);
    }
  }

  @ReactMethod
  fun getText( promise: Promise) {
    Log.v(logTag, "getText")
    try {
      val text = DocumentDataReader.getText()
      promise.resolve(text)
    }
    catch(ex:Exception )
    {
      Log.e(logTag, "getText Exception"+ ex)
      promise.reject(ex);
    }
  }

  @ReactMethod
  fun analyze( promise: Promise) {
    Log.v(logTag, "analyze")
    try {
      val text = DocumentDataReader.analyze()
      Log.d(logTag, "analyze:" + text)

      promise.resolve(text)
    }
    catch(ex:Exception )
    {
      Log.e(logTag, "analyze Exception"+ ex)
      promise.reject(ex);
    }
  }

  companion object {
    const val NAME = "DocutainSdk"
    var licenseKeyMissing = false
  }

  private fun getLogLevelFromString(loglevel: String) : Logger.Level {
    when (loglevel.uppercase()) {
      "DISABLE" -> return Logger.Level.DISABLE;
      "ASSERT" -> return Logger.Level.ASSERT;
      "ERROR" -> return Logger.Level.ERROR;
      "WARNING" -> return Logger.Level.WARNING;
      "INFO" -> return Logger.Level.INFO;
      "DEBUG" -> return Logger.Level.DEBUG;
      "VERBOSE" -> return Logger.Level.VERBOSE;
      else -> return Logger.Level.ERROR;
      }
  }

  private fun getScanFilterFromString(scanFilter: String) : ScanFilter {
    when (scanFilter.uppercase()) {
      "AUTO" -> return ScanFilter.AUTO;
      "GRAY" -> return ScanFilter.GRAY;
      "BLACKWHITE" -> return ScanFilter.BLACKWHITE;
      "ORIGINAL" -> return ScanFilter.ORIGINAL;
      "TEXT" -> return ScanFilter.TEXT;
      "AUTO2" -> return ScanFilter.AUTO2;
      "ILLUSTRATION" -> return ScanFilter.ILLUSTRATION;
      else -> return ScanFilter.ILLUSTRATION;
    }
  }

  private fun getScanSourceFromString(scanSource: String) : Source {
    when (scanSource.uppercase()) {
      "CAMERA" -> return Source.CAMERA;
      "IMAGE" -> return Source.IMAGE;
      "GALLERY" -> return Source.GALLERY;
      "GALLERY_MULTIPLE" -> return Source.GALLERY_MULTIPLE;
      else -> return Source.CAMERA;
    }
  }

  private fun getPDFPageFormatFromString(pageFormat: String) : Document.PDFPageFormat {
    when (pageFormat.uppercase()) {
      "FIT_TO_PAGES" -> return Document.PDFPageFormat.FIT_TO_PAGES;
      "A4" -> return Document.PDFPageFormat.A4;
      "A4_LANDSCAPE" -> return Document.PDFPageFormat.A4_LANDSCAPE;
      "A5" -> return Document.PDFPageFormat.A5;
      "A5_LANDSCAPE" -> return Document.PDFPageFormat.A5_LANDSCAPE;
      "LETTER" -> return Document.PDFPageFormat.LETTER;
      "LETTER_LANDSCAPE" -> return Document.PDFPageFormat.LETTER_LANDSCAPE;
      "LEGAL" -> return Document.PDFPageFormat.LEGAL;
      "LEGAL_LANDSCAPE" -> return Document.PDFPageFormat.LEGAL_LANDSCAPE;
      else -> return Document.PDFPageFormat.A4;
    }
  }

  private fun getPageSourceTypeFromString(pageSourceType: String) : Document.PageSourceType {
    when (pageSourceType.uppercase()) {
      "ORIGINAL" -> return Document.PageSourceType.ORIGINAL;
      "CUT_FILTER" -> return Document.PageSourceType.CUT_FILTER;
      "CUT_ONLY" -> return Document.PageSourceType.CUT_ONLY;
      else -> return Document.PageSourceType.CUT_FILTER;
    }
  }


  private fun mapAnalyzeConfiguration(config: ReadableMap, analyzeConfig: AnalyzeConfiguration) {

    if(config.hasKey("readBIC"))
      analyzeConfig.readBIC = config.getBoolean("readBIC")
    if(config.hasKey("readPaymentState"))
      analyzeConfig.readPaymentState = config.getBoolean("readPaymentState")
  }
  private fun mapDocumentScannerConfiguration(options: ReadableMap, scanConfig: DocumentScannerConfiguration) {
    if(options.hasKey("pageEditConfig")) {
      val optionsPallowCaptureModeSetting = options.getMap("pageEditConfig")
      if(optionsPallowCaptureModeSetting != null)
      {
        if(optionsPallowCaptureModeSetting.hasKey("allowPageFilter")) {
          scanConfig.pageEditConfig.allowPageFilter =
            optionsPallowCaptureModeSetting.getBoolean("allowPageFilter")
        }
        if(optionsPallowCaptureModeSetting.hasKey("allowPageRotation")) {
          scanConfig.pageEditConfig.allowPageRotation = optionsPallowCaptureModeSetting.getBoolean("allowPageRotation")
        }
        if(optionsPallowCaptureModeSetting.hasKey("allowPageArrangement")) {
          scanConfig.pageEditConfig.allowPageArrangement = optionsPallowCaptureModeSetting.getBoolean("allowPageArrangement")
        }
        if(optionsPallowCaptureModeSetting.hasKey("allowPageCropping")) {
          scanConfig.pageEditConfig.allowPageCropping = optionsPallowCaptureModeSetting.getBoolean("allowPageCropping")
        }
        if(optionsPallowCaptureModeSetting.hasKey("pageArrangementShowDeleteButton")) {
          scanConfig.pageEditConfig.pageArrangementShowDeleteButton = optionsPallowCaptureModeSetting.getBoolean("pageArrangementShowDeleteButton")
        }
        if(optionsPallowCaptureModeSetting.hasKey("pageArrangementShowPageNumber")) {
          scanConfig.pageEditConfig.pageArrangementShowPageNumber = optionsPallowCaptureModeSetting.getBoolean("pageArrangementShowPageNumber")
        }
      }
    }

    try {
    if(options.hasKey("ColorConfig")) {
      val optionsColorConfig = options.getMap("ColorConfig")
       val iterator = optionsColorConfig!!.keySetIterator()
        while (iterator.hasNextKey()) {
          val key = iterator.nextKey()
          val docutainColorEntry = optionsColorConfig.getMap(key)
          if(docutainColorEntry!=null){
            val light = docutainColorEntry.getString("Light")
            val dark = docutainColorEntry.getString("Dark")
            if(light != null && dark != null) {
               val docutainColor = DocutainColor(light, dark)
              Log.v("DocutainSdkFull", key + " Light:" + light + " Dark:" + dark)
              when (key.uppercase()) {
                "COLORPRIMARY" -> scanConfig.colorConfig.colorPrimary = docutainColor
                "COLORSECONDARY" -> scanConfig.colorConfig.colorSecondary = docutainColor
                "COLORONSECONDARY" -> scanConfig.colorConfig.colorOnSecondary = docutainColor
                "COLORSCANBUTTONSLAYOUTBACKGROUND" -> scanConfig.colorConfig.colorScanButtonsLayoutBackground = docutainColor
                "COLORSCANBUTTONSFOREGROUND" -> scanConfig.colorConfig.colorScanButtonsForeground = docutainColor
                "COLORSCANPOLYGON" -> scanConfig.colorConfig.colorScanPolygon = docutainColor
                "COLORBOTTOMBARBACKGROUND" -> scanConfig.colorConfig.colorBottomBarBackground = docutainColor
                "COLORBOTTOMBARFOREGROUND" -> scanConfig.colorConfig.colorBottomBarForeground = docutainColor
                "COLORTOPBARBACKGROUND" -> scanConfig.colorConfig.colorTopBarBackground = docutainColor
                "COLORTOPBARFOREGROUND" -> scanConfig.colorConfig.colorTopBarForeground = docutainColor
                else -> Log.e("DocutainSdkFull", "mapDocumentScannerConfiguration color not valid "+ key)
              }
            }
          }
        }
    }

    if(options.hasKey("allowCaptureModeSetting"))
      scanConfig.allowCaptureModeSetting = options.getBoolean("allowCaptureModeSetting")
    if(options.hasKey("autoCapture"))
      scanConfig.autoCapture = options.getBoolean("autoCapture")
    if(options.hasKey("onboardingImageSource")){
      val imgSource = options.getString("onboardingImageSource")!!
      val imgIdentifier = DocutainSDK.context.resources.getIdentifier(imgSource, "drawable", DocutainSDK.context.packageName)
      scanConfig.onboardingImageSource = imgIdentifier
    }
    if(options.hasKey("defaultScanFilter"))
      scanConfig.defaultScanFilter = getScanFilterFromString(options.getString("defaultScanFilter")!!)
    if(options.hasKey("source"))
      scanConfig.source = getScanSourceFromString(options.getString("source")!!)
    if(options.hasKey("sourceImages")){     
      val items = options.getArray("sourceImages")!! as ReadableArray
      val sourceImages = ArrayList<java.io.File>()
      for (i in 0..items.size()-1) {
          sourceImages.add(java.io.File(items.getString(i)))
      }
      scanConfig.sourceImages = sourceImages
    }      
    if(options.hasKey("autoCrop"))
      scanConfig.autoCrop = options.getBoolean("autoCrop")
    if(options.hasKey("multiPage"))
      scanConfig.multiPage = options.getBoolean("multiPage")
    if(options.hasKey("preCaptureFocus"))
      scanConfig.preCaptureFocus = options.getBoolean("preCaptureFocus")
    }    
    catch(ex:Exception )
    {
      Log.e("DocutainSdkFull", "mapDocumentScannerConfiguration Exception"+ ex)
    }
  }
}
