/** 
*   Docutain SDK React Native
*   Copyright (c) INFOSOFT Informations- und Dokumentationssysteme GmbH. All rights reserved.
*
*   Docutain SDK React Native is a commercial product and requires a license. 
*   Details found in the LICENSE file in the root directory of this source tree.
*/

import DocutainSdk
import typealias DocutainSdk.Logger
import typealias DocutainSdk.Document
import typealias DocutainSdk.Source
import typealias DocutainSdk.ScanFilter

@objc(DocutainSdk)
class DocutainSdk: NSObject, ScanDelegate {

private var scanResolve : RCTPromiseResolveBlock? = nil

  @objc(multiply:withB:withResolver:withRejecter:)
  func multiply(a: Float, b: Float, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void {
    resolve(a*b)
  }

  @objc(initSDK:withResolver:withRejecter:)
  func initSDK(licenseKey: String, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
    if(!DocutainSDK.initSDK(licenseKey: licenseKey)){
      resolve(false)
    } else{
      resolve(true)
    }
  }

  @objc(getLastError:withRejecter:)
  func getLastError(resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
    resolve(DocutainSDK.getLastError())
  }

  @objc(setAnalyzeConfiguration:withResolver:withRejecter:)
  func setAnalyzeConfiguration(config: [String: Any], resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
    //only available for iOS >= 13
    if #available(iOS 13, *){
      let analyzeConfig = AnalyzeConfiguration()
      mapAnalyzeConfiguration(config: config, analyzeConfig: analyzeConfig)
      if(!DocumentDataReader.setAnalyzeConfiguration(analyzeConfiguration: analyzeConfig)){
        resolve(false)
      } else{
        resolve(true)
      }
    } else{
      resolve(false)
    }
  }

  @objc(scanDocument:withResolver:withRejecter:)
  func scanDocument(config: [String: Any], resolve:@escaping RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
    let scanConfig = DocumentScannerConfiguration()
    mapDocumentScannerConfiguration(config: config, scanConfig: scanConfig)
    scanResolve = resolve
    //react runs async, so we need to dispatch on ui thread
    DispatchQueue.main.async {
        UI.scanDocument(scanDelegate: self, scanConfig: scanConfig)
    }
  }

  @objc(loadFile:withResolver:withRejecter:)
  func loadFile(path: String, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
      //only available for iOS >= 13
      if #available(iOS 13, *){
          if let url = URL(string: path){
              if(!DocumentDataReader.loadFile(fileUrl: url as URL)){
                resolve(false)
              } else{
                resolve(true)
              }
          } else{
              resolve(false)
          }
      } else{
          resolve(false)
      }
  }

  @objc(writePDF:withOverwrite:withPageFormat:withResolver:withRejecter:)
  func writePDF(filepath: String, overWrite: Bool, pageFormat: String, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
      if let url = URL(string: filepath){
          if let returnURL = Document.writePDF(fileUrl: url.deletingLastPathComponent(), fileName: url.lastPathComponent, overwrite: overWrite, pageFormat: getPDFPageFormatFromString(pageFormat: pageFormat)){
          resolve(returnURL.path)
        } else{
          resolve("")
        }
      } else{
        resolve("")
      }
  }

  @objc(writeImage:withPath:withResolver:withRejecter:)
  func writeImage(pageNumber: Int, filepath: String, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
      if let url = URL(string: filepath){
          if let returnURL = Document.writeImage(page: pageNumber, fileUrl: url){
          resolve(returnURL.path)
        } else{
          resolve("")
        }
      } else{
        resolve("")
      }
  }
    
  @objc(getImageBytes:withPageSourceType:withResolver:withRejecter:)
  func getImageBytes(pageNumber: Int, pageSourceType: String, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
    if let data = Document.getImage(page: pageNumber, pageSourceType: getPageSourceTypeFromString(pageSourceType: pageSourceType)){
        resolve(data.base64EncodedString())
    } else{
        resolve("")
      }
  }

  @objc(getText:withRejecter:)
  func getText(resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
      //only available for iOS >= 13
      if #available(iOS 13, *){
          resolve(DocumentDataReader.getText())
      } else{
          resolve("")
      }
  }

  @objc(analyze:withRejecter:)
  func analyze(resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
      //only available for iOS >= 13
      if #available(iOS 13, *){
          resolve(DocumentDataReader.analyze())
      } else{
          resolve("")
      }
  }

  @objc(setLogLevel:withResolver:withRejecter:)
  func setLogLevel(logLevel: String, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock)-> Void{
      Logger.setLogLevel(level: getLogLevelFromString(loglevel: logLevel))
      resolve(true)
  }

  @objc(getTraceFile:withRejecter:)
  func getTraceFile(resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock)-> Void{
    resolve(Logger.getTraceFile())
  }

  @objc(deleteTempFiles:withResolver:withRejecter:)
  func deleteTempFiles(deleteTraceFileContent:Bool,resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock)-> Void{
    resolve(DocutainSDK.deleteTempFiles(deleteTraceFileContent:deleteTraceFileContent))
  }

  internal func didFinishScan(withResult result: Bool) {
    if let resolve = scanResolve{
      resolve(result)
    } 
    //reset
    scanResolve = nil
  }

  private func mapAnalyzeConfiguration(config: [String: Any], analyzeConfig: AnalyzeConfiguration) -> Void{
    if let readBIC = config["readBIC"] as? Bool{
      analyzeConfig.readBIC = readBIC
    }
    if let readPaymentState = config["readPaymentState"] as? Bool{
      analyzeConfig.readPaymentState = readPaymentState
    }
  }

  private func mapDocumentScannerConfiguration(config: [String: Any], scanConfig: DocumentScannerConfiguration) -> Void{
    if let onboardingImageSource = config["onboardingImageSource"] as? String{
      scanConfig.onboardingImageSource = onboardingImageSource
    }
    if let allowCaptureModeSetting = config["allowCaptureModeSetting"] as? Bool{
      scanConfig.allowCaptureModeSetting = allowCaptureModeSetting
    }
    if let autoCapture = config["autoCapture"] as? Bool{
      scanConfig.autoCapture = autoCapture
    }
    if let defaultScanFilter = config["defaultScanFilter"] as? String{
      scanConfig.defaultScanFilter = getScanFilterFromString(scanFilter: defaultScanFilter)
    }
    if let source = config["source"] as? String{
      scanConfig.source = getScanSourceFromString(scanSource: source)
    }
    if let sourceImages = config["sourceImages"] as? [String]{
      var images: [URL] = []
      for image in sourceImages{
        images.append(URL(fileURLWithPath:image))
      }
      scanConfig.sourceImages = images
    }
    if let autoCrop = config["autoCrop"] as? Bool{
      scanConfig.autoCrop = autoCrop
    }
    if let multiPage = config["multiPage"] as? Bool{
      scanConfig.multiPage = multiPage
    }
    
    if let pageEditConfig = config["pageEditConfig"] as? [String: Any]{
      if let allowPageFilter = pageEditConfig["allowPageFilter"] as? Bool {
        scanConfig.pageEditConfig.allowPageFilter = allowPageFilter
      }
      if let allowPageRotation = pageEditConfig["allowPageRotation"] as? Bool {
        scanConfig.pageEditConfig.allowPageRotation = allowPageRotation
      }
      if let allowPageArrangement = pageEditConfig["allowPageArrangement"] as? Bool {
        scanConfig.pageEditConfig.allowPageArrangement = allowPageArrangement
      }
      if let allowPageCropping = pageEditConfig["allowPageCropping"] as? Bool {
        scanConfig.pageEditConfig.allowPageCropping = allowPageCropping
      }
      if let pageArrangementShowDeleteButton = pageEditConfig["pageArrangementShowDeleteButton"] as? Bool {
        scanConfig.pageEditConfig.pageArrangementShowDeleteButton = pageArrangementShowDeleteButton
      }
      if let pageArrangementShowPageNumber = pageEditConfig["pageArrangementShowPageNumber"] as? Bool {
        scanConfig.pageEditConfig.pageArrangementShowPageNumber = pageArrangementShowPageNumber
      }
    }
    if let colorConfig = config["ColorConfig"] as? [String: Any] {
        for (key, value) in colorConfig {
            if let color = value as? [String: Any]{
                if let colorLight = color["Light"] as? String, let colorDark = color["Dark"] as? String{
                    switch(key.uppercased()){
                    case "COLORPRIMARY" :
                        scanConfig.colorConfig.setColorPrimary(light: UIColor.init(hexaString: colorLight), dark: UIColor.init(hexaString: colorDark))
                    case "COLORSECONDARY" :
                        scanConfig.colorConfig.setColorSecondary(light: UIColor.init(hexaString: colorLight), dark: UIColor.init(hexaString: colorDark))
                    case "COLORONSECONDARY" :
                        scanConfig.colorConfig.setColorOnSecondary(light: UIColor.init(hexaString: colorLight), dark: UIColor.init(hexaString: colorDark))
                    case "COLORSCANBUTTONSLAYOUTBACKGROUND" :
                        scanConfig.colorConfig.setColorScanButtonsLayoutBackground(light: UIColor.init(hexaString: colorLight), dark: UIColor.init(hexaString: colorDark))
                    case "COLORSCANBUTTONSFOREGROUND" :
                        scanConfig.colorConfig.setColorScanButtonsForeground(light: UIColor.init(hexaString: colorLight), dark: UIColor.init(hexaString: colorDark))
                    case "COLORSCANPOLYGON" :
                        scanConfig.colorConfig.setColorScanPolygon(light: UIColor.init(hexaString: colorLight), dark: UIColor.init(hexaString: colorDark))
                    case "COLORBOTTOMBARBACKGROUND" :
                        scanConfig.colorConfig.setColorBottomBarBackground(light: UIColor.init(hexaString: colorLight), dark: UIColor.init(hexaString: colorDark))
                    case "COLORBOTTOMBARFOREGROUND" :
                        scanConfig.colorConfig.setColorBottomBarForeground(light: UIColor.init(hexaString: colorLight), dark: UIColor.init(hexaString: colorDark))
                    case "COLORTOPBARBACKGROUND" :
                        scanConfig.colorConfig.setColorTopBarBackground(light: UIColor.init(hexaString: colorLight), dark: UIColor.init(hexaString: colorDark))
                    case "COLORTOPBARFOREGROUND" :
                        scanConfig.colorConfig.setColorTopBarForeground(light: UIColor.init(hexaString: colorLight), dark: UIColor.init(hexaString: colorDark))
                    default:
                        print("mapDocumentScannerConfiguration color not valid " + key)
                    }
                }
            }
        }
    }
  }

  private func getLogLevelFromString(loglevel: String) -> Logger.Level {
    switch (loglevel.uppercased()) {
    case "DISABLE":
        return Logger.Level.disable
    case "ASSERT":
        return Logger.Level.assert
    case "ERROR":
        return Logger.Level.error
    case "WARNING":
        return Logger.Level.warning
    case "INFO":
        return Logger.Level.info
    case "DEBUG":
        return Logger.Level.debug
    case "VERBOSE":
        return Logger.Level.verbose
      default:
        return Logger.Level.error
      }
  }

  private func getScanFilterFromString(scanFilter: String) -> ScanFilter {
    switch (scanFilter.uppercased()) {
      case "AUTO":
        return ScanFilter.auto
      case "GRAY":
        return ScanFilter.gray
      case "BLACKWHITE":
        return ScanFilter.blackWhite
      case "ORIGINAL":
        return ScanFilter.original
      case "TEXT":
        return ScanFilter.text
      case "AUTO2":
        return ScanFilter.auto2
      case "ILLUSTRATION":
        return ScanFilter.illustration
      default:
        return ScanFilter.illustration
    }
  }

  private func getScanSourceFromString(scanSource: String) -> Source {
    switch (scanSource.uppercased()) {
      case "CAMERA":
        return Source.camera
      case "IMAGE":
        return Source.image
      case "GALLERY":
        return Source.gallery
      case "GALLERY_MULTIPLE":
      if #available(iOS 14, *){
        return Source.galleryMultiple
      } else{
        //fallback to single image picking
        return Source.gallery
      }
      default:
        return Source.camera
    }
  }

  private func getPDFPageFormatFromString(pageFormat: String) -> Document.PDFPageFormat {
    switch (pageFormat.uppercased()) {
      case "FIT_TO_PAGES":
        return Document.PDFPageFormat.FitToPages
      case "A4":
        return Document.PDFPageFormat.A4
      case "A4_LANDSCAPE":
        return Document.PDFPageFormat.A4Landscape
      case "A5":
        return Document.PDFPageFormat.A5
      case "A5_LANDSCAPE":
        return Document.PDFPageFormat.A5Landscape
      case "LETTER":
        return Document.PDFPageFormat.Letter
      case "LETTER_LANDSCAPE":
        return Document.PDFPageFormat.LetterLandscape
      case "LEGAL":
        return Document.PDFPageFormat.Legal
      case "LEGAL_LANDSCAPE":
        return Document.PDFPageFormat.LegalLandscape
      default:
        return Document.PDFPageFormat.A4
    }
  }
    
    private func getPageSourceTypeFromString(pageSourceType: String) -> Document.PageSourceType{
        switch (pageSourceType.uppercased()) {
          case "ORIGINAL":
            return Document.PageSourceType.original
          case "CUT_FILTER":
            return Document.PageSourceType.cutFilter
          case "CUT_ONLY":
            return Document.PageSourceType.cutOnly
          default:
            return Document.PageSourceType.cutFilter
        }
    }
}

private extension UIColor {
    convenience init(hexaString: String, alpha: CGFloat = 1) {
        let chars = Array(hexaString.dropFirst())
        self.init(red:   .init(strtoul(String(chars[0...1]),nil,16))/255,
                  green: .init(strtoul(String(chars[2...3]),nil,16))/255,
                  blue:  .init(strtoul(String(chars[4...5]),nil,16))/255,
                  alpha: alpha)}
}
